#include <amxmodx>
#include <amxmisc>
#include <ze_core>
#include <easy_http>

//#pragma dynamic 32768

// Defines.
#define MAX_URL_LENGTH  2000
#define MAX_JSON_LENGTH 2048   // 2kb

#define MENU_ITEM_OTHER '_'

#define REPORT_ID    // Remove // to enable Report ID element.

// File Name.
new const FILE_NAME[] = "ReportTemplate"

// Log Name.
new const LOG_NAME[] = "DiscordReportSystem.log"

// Cvars.
new g_iMaxTimes,
	bool:g_bEnabled,
	Float:g_flDelayTime

// Array.
new g_iUser[MAX_PLAYERS+1],
	g_iTimes[MAX_PLAYERS+1],
	Float:g_flDelay[MAX_PLAYERS+1]

// String.
new g_szContent[MAX_JSON_LENGTH],
	g_szWebHookURL[MAX_URL_LENGTH]

// Trie's.
new Trie:g_tTimes,
	Trie:g_tDelay

// Dynamic Array.
new Array:g_aReasons

public plugin_init()
{
	// Load Plug-In.
	register_plugin("[ZE] Addon: Discord Report System", "1.0.1", ZE_AUTHORS)

	// Cvars.
	bind_pcvar_num(register_cvar("ze_reportdc_enable", "1"), g_bEnabled)
	bind_pcvar_num(register_cvar("ze_reportdc_times", "3"), g_iMaxTimes)
	bind_pcvar_float(register_cvar("ze_reportdc_cooldown", "60.0"), g_flDelayTime)

	// Commands.
	register_clcmd("say /rpt", "cmd_ReportMenu")
	register_clcmd("say /report", "cmd_ReportMenu")
	register_clcmd("say_team /rpt", "cmd_ReportMenu")
	register_clcmd("say_team /report", "cmd_ReportMenu")

	register_clcmd("WHATS_REASON", "cmd_EnterReason")

	// Create dyn Array.
	g_aReasons = ArrayCreate(64, 1)

	// Create Hash String (Trie's).
	g_tTimes = TrieCreate()
	g_tDelay = TrieCreate()
}

public plugin_end()
{
	// Free the Memory.
	TrieDestroy(g_tTimes)
	TrieDestroy(g_tDelay)
	ArrayDestroy(g_aReasons)
}

public plugin_cfg()
{
	new const szReasons[][] = {"Cheats/Hack", "Insult/Abuse", "Spoil the gameplay", "Teamplay", "Chat/Voice Spammer"}

	// Read 'Report Reasons' from INI file.
	ini_read_string_array(ZE_FILENAME, "DC Report System", "REPORT_REASONS", g_aReasons)

	if (!ArraySize(g_aReasons))
	{
		for (new i = 0; i < sizeof(szReasons); i++)
			ArrayPushString(g_aReasons, szReasons[i])

		ini_write_string_array(ZE_FILENAME, "DC Report System", "REPORT_REASONS", g_aReasons)
	}

	if (!ini_read_string(ZE_FILENAME, "DC Report System", "WEBHOOK_URL", g_szWebHookURL, charsmax(g_szWebHookURL)))
		ini_write_string(ZE_FILENAME, "DC Report System", "WEBHOOK_URL", g_szWebHookURL)

	new szDataDir[24], szFile[64]
	get_datadir(szDataDir, charsmax(szDataDir))
	formatex(szFile, charsmax(szFile), "%s/%s.json", szDataDir, FILE_NAME)
	read_File(szFile)
}

public client_authorized(id, const szAuthID[])
{
	// HLTV Proxy?
	if (is_user_hltv(id))
		return

	if (TrieGetCell(g_tTimes, szAuthID, g_iTimes[id]))
		TrieDeleteKey(g_tTimes, szAuthID)
	if (TrieGetCell(g_tDelay, szAuthID, g_flDelay[id]))
		TrieDeleteKey(g_tDelay, szAuthID)
}

public client_disconnected(id, bool:drop, message[], maxlen)
{
	// HLTV Proxy?
	if (is_user_hltv(id))
		return

	new szAuthID[MAX_AUTHID_LENGTH]
	get_user_authid(id, szAuthID, charsmax(szAuthID))
	TrieSetCell(g_tTimes, szAuthID, g_iTimes[id])
	TrieSetCell(g_tDelay, szAuthID, g_flDelay[id])

	g_iUser[id] = 0
	g_iTimes[id] = 0
	g_flDelay[id] = 0.0
}

public cmd_ReportMenu(id)
{
	if (!g_bEnabled)
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "CMD_NOT")
		return PLUGIN_CONTINUE
	}

	if (!g_szWebHookURL[0])
	{
		ze_colored_print(id, "Error : Webhook URL not found !")  // Error : No need translation!
		return PLUGIN_HANDLED_MAIN
	}

	static Float:flHlTime; flHlTime = get_gametime()

	if (g_flDelayTime > 0.0 && g_flDelay[id] > flHlTime)
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "MSG_REPORT_DELAY", g_flDelay[id] - flHlTime)
		return PLUGIN_CONTINUE
	}

	if (g_iMaxTimes > 0 && g_iTimes[id] >= g_iMaxTimes)
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "MSG_REPORT_LIMIT", g_iMaxTimes)
		return PLUGIN_CONTINUE
	}

	show_Report_Menu(id)
	return PLUGIN_CONTINUE
}

public cmd_EnterReason(id)
{
	new iReported
	if ((iReported = g_iUser[id]) == 0)
		return PLUGIN_HANDLED

	// Player disconnected?
	if (!is_user_connected(iReported))
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "CMD_NOT")
		return PLUGIN_HANDLED_MAIN
	}

	new szArgs[128]
	read_args(szArgs, charsmax(szArgs))
	remove_quotes(szArgs), trim(szArgs)

	if (!szArgs[0] || strlen(szArgs) < 4)
	{
		client_cmd(id, "messagemode WHATS_REASON")
		return PLUGIN_HANDLED_MAIN
	}

	g_iUser[id] = 0
	initial_ReportMessage(id, iReported, szArgs)
	return PLUGIN_HANDLED_MAIN
}

public show_Report_Menu(id)
{
	// Title:
	new iMenu = menu_create(fmt("\r%L \y%L", LANG_PLAYER, "MENU_PREFIX", LANG_PLAYER, "MENU_REPORT_TITLE"), "handler_Report_Menu")

	new iPlayers[MAX_PLAYERS], iPlayersNum
	get_players(iPlayers, iPlayersNum, "h")

	for (new szName[MAX_NAME_LENGTH], iItemData[2], player, i = 0; i < iPlayersNum; i++)
	{
		player = iPlayers[i]

		if (id == player)
			continue

		iItemData[0] = player
		get_user_name(player, szName, charsmax(szName))
		menu_additem(iMenu, szName, iItemData, 0)
	}

	if (!menu_items(iMenu))
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "CMD_NOT")
		menu_destroy(iMenu)
		return
	}

	menu_addtext2(iMenu, fmt("^n\r%L", LANG_PLAYER, "MENU_REPORT_WARN"))

	// Next, Back, Exit
	menu_setprop(iMenu, MPROP_NEXTNAME, fmt("%L", LANG_PLAYER, "MENU_NEXT"))
	menu_setprop(iMenu, MPROP_BACKNAME, fmt("%L", LANG_PLAYER, "MENU_BACK"))
	menu_setprop(iMenu, MPROP_EXITNAME, fmt("%L", LANG_PLAYER, "MENU_EXIT"))

	menu_display(id, iMenu, 0, 30)
}

public handler_Report_Menu(id, iMenu, iKey)
{
	switch (iKey)
	{
		case MENU_TIMEOUT, MENU_EXIT:
			goto closeMenu
	}

	new iItemData[2]
	menu_item_getinfo(iMenu, iKey, _, iItemData, charsmax(iItemData))
	new const player = iItemData[0]

	// Player disconnected?
	if (!player || !is_user_connected(player))
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "CMD_NOT")
		goto closeMenu
	}

	g_iUser[id] = player
	show_Reasons_Menu(id)

	// Free the Memory.
	closeMenu:
	menu_destroy(iMenu)
	return PLUGIN_HANDLED
}

public show_Reasons_Menu(id)
{
	new const iSizeNum = ArraySize(g_aReasons)

	if (!iSizeNum)
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "CMD_NOT")
		return
	}

	new szName[MAX_NAME_LENGTH]
	get_user_name(g_iUser[id], szName, charsmax(szName))

	// Title:
	new iMenu = menu_create(fmt("\r%L \y%L", LANG_PLAYER, "MENU_PREFIX", LANG_PLAYER, "MENU_REASONS_TITLE", szName), "handler_Reasons_Menu")

	new iItemData[2]
	for (new szReason[64], i = 0; i < iSizeNum; i++)
	{
		ArrayGetString(g_aReasons, i, szReason, charsmax(szReason))

		iItemData[0] = i
		menu_additem(iMenu, szReason, iItemData)
	}

	iItemData[0] = MENU_ITEM_OTHER
	menu_additem(iMenu, fmt("%L", LANG_PLAYER, "MENU_REASON_OTHER"), iItemData)

	// Next, Back, Exit
	menu_setprop(iMenu, MPROP_NEXTNAME, fmt("%L", LANG_PLAYER, "MENU_NEXT"))
	menu_setprop(iMenu, MPROP_BACKNAME, fmt("%L", LANG_PLAYER, "MENU_BACK"))
	menu_setprop(iMenu, MPROP_EXITNAME, fmt("%L", LANG_PLAYER, "MENU_EXIT"))

	menu_display(id, iMenu, 0, 30)
}

public handler_Reasons_Menu(id, iMenu, iKey)
{
	switch (iKey)
	{
		case MENU_TIMEOUT, MENU_EXIT:
			goto closeMenu
	}

	new iItemData[2]
	menu_item_getinfo(iMenu, iKey, _, iItemData, charsmax(iItemData))
	new const item = iItemData[0]
	new const player = g_iUser[id]

	// Player disconnected?
	if (!player || !is_user_connected(player))
	{
		ze_colored_print(id, "%L", LANG_PLAYER, "CMD_NOT")
		goto closeMenu
	}

	if (item == MENU_ITEM_OTHER)
	{
		client_cmd(id, "messagemode WHATS_REASON")
	}
	else
	{
		new szReason[64]
		ArrayGetString(g_aReasons, item, szReason, charsmax(szReason))
		initial_ReportMessage(id, player, szReason)
	}

	// Free the Memory.
	closeMenu:
	menu_destroy(iMenu)
	return PLUGIN_HANDLED
}

public initial_ReportMessage(iSender, iReported, const szReason[])
{
	new szContent[MAX_JSON_LENGTH], szLogFile[256], szName[MAX_NAME_LENGTH], szAuthID[MAX_AUTHID_LENGTH], szIPAddr[MAX_IP_WITH_PORT_LENGTH], iLen
	copy(szContent, charsmax(szContent), g_szContent)

	// Sender.
	get_user_name(iSender, szName, charsmax(szName))
	get_user_authid(iSender, szAuthID, charsmax(szAuthID))
	get_user_ip(iSender, szIPAddr, charsmax(szIPAddr))
	replace_string(szContent, charsmax(szContent), "{$sender_name}", szName)
	replace_string(szContent, charsmax(szContent), "{$sender_authid}", szAuthID)
	replace_string(szContent, charsmax(szContent), "{$sender_ipaddr}", szIPAddr)
	iLen += formatex(szLogFile[iLen], charsmax(szLogFile) - iLen, "Sender: [Name: %s - AuthID: %s - IP Address: %s] ", szName, szAuthID, szIPAddr)

	// Report ID.
	#if defined REPORT_ID
		new szReportID[34]
		hash_string(fmt("%s_%08d", szName, random(99999999)), Hash_Crc32, szReportID, charsmax(szReportID))
		replace_string(szContent, charsmax(szContent), "{$report_id}", szReportID)
	#endif

	// Reported.
	get_user_name(iReported, szName, charsmax(szName))
	get_user_authid(iReported, szAuthID, charsmax(szAuthID))
	get_user_ip(iReported, szIPAddr, charsmax(szIPAddr))
	replace_string(szContent, charsmax(szContent), "{$target_name}", szName)
	replace_string(szContent, charsmax(szContent), "{$target_authid}", szAuthID)
	replace_string(szContent, charsmax(szContent), "{$target_ipaddr}", szIPAddr)
	iLen += formatex(szLogFile[iLen], charsmax(szLogFile) - iLen, "Reported: [Name: %s - AuthID: %s - IP Address: %s] ", szName, szAuthID, szIPAddr)

	new bool:bIsAdmin = is_user_admin(iReported) ? true : false

	// Reason/Is Admin.
	replace_string(szContent, charsmax(szContent), "{$reason}", szReason)
	replace_string(szContent, charsmax(szContent), "{$is_admin}", bIsAdmin ? ":green_circle:" : ":red_circle:")

	if (bIsAdmin)
		log_to_file(LOG_NAME, "(ID: %s) Report on Admin >> %s", szReportID, szLogFile)
	else
		log_to_file(LOG_NAME, "(ID: %s) Report on Player >> %s", szReportID, szLogFile)

	if (g_flDelayTime > 0.0)
		g_flDelay[iSender] = get_gametime() + g_flDelayTime

	if (g_iMaxTimes > 0)
		g_iTimes[iSender]++

	g_iUser[iSender] = 0
	send_Report(szContent)
	ze_colored_print(iSender, "%L", LANG_PLAYER, "MSG_REPORT_SENT")
}

// Credit: Sam (sh_iva)
public send_Report(const szContent[])
{
	new EzHttpOptions:hOpt = ezhttp_create_options()
	ezhttp_option_set_header(hOpt, "Content-Type", "application/json")
	ezhttp_option_set_header(hOpt, "User-Agent", "AMXX-Report-System")
	ezhttp_option_set_connect_timeout(hOpt, 10 * 1000)
	ezhttp_option_set_timeout(hOpt, 10 * 1000)
	ezhttp_option_set_body(hOpt, szContent)
	ezhttp_post(g_szWebHookURL, "requestComplete", hOpt)
}

public requestComplete(EzHttpRequest:iRequest)
{
	if (ezhttp_get_error_code(iRequest) != EZH_OK)
	{
		new szError[64]
		ezhttp_get_error_message(iRequest, szError, charsmax(szError))
		server_print("Reponse Error : %s", szError)
	}
}

/**
 * -=| Function |=-
 */
read_File(const szFile[])
{
	new hFile  // Open the file.
	if ((hFile = fopen(szFile, "rt")))
	{
		new szRead[512], iLen

		while (!feof(hFile))
		{
			szRead = NULL_STRING

			if (!fgets(hFile, szRead, charsmax(szRead)))
				continue

			trim(szRead), replace(szRead, charsmax(szRead), "^n", "")

			if (szRead[0] == EOS)
				continue

			iLen += copy(g_szContent[iLen], charsmax(g_szContent) - iLen, szRead)
		}

		// Close the file.
		fclose(hFile)
		return 1
	}

	return 0
}